@@ -3,6 +3,9 @@
 # Distributed under the MIT software license
 # TODO: Test EBMUtils
 
+from math import ceil
+from .internal import Native, Booster, InteractionDetector
+
 # from scipy.special import expit
 from sklearn.utils.extmath import softmax
 from sklearn.model_selection import train_test_split
@@ -12,6 +15,8 @@ import numpy as np
 import warnings
 import copy
 
+from scipy.stats import norm
+from scipy.optimize import root_scalar, brentq
 
 import logging
 
@@ -264,34 +269,68 @@ class EBMUtils:
         # all test/train splits should be done with this function to ensure that
         # if we re-generate the train/test splits that they are generated exactly
         # the same as before
+
+        if (X.shape[0] != len(y) or X.shape[0] != len(w)):
+            raise Exception("Data, labels and weights should have the same number of rows.")
+
+        sampling_result = None
+
         if test_size == 0:
             X_train, y_train, w_train = X, y, w
             X_val = np.empty(shape=(0, X.shape[1]), dtype=X.dtype)
             y_val = np.empty(shape=(0,), dtype=y.dtype)
             w_val = np.empty(shape=(0,), dtype=w.dtype)
+
         elif test_size > 0:
+            n_samples = X.shape[0]
+            n_test_samples = 0
+
+            if test_size >= 1:
+                if test_size % 1:
+                    raise Exception("If test_size >= 1, test_size should be a whole number.")
+                n_test_samples = test_size 
+            else:
+                n_test_samples = ceil(n_samples * test_size)
+
+            n_train_samples = n_samples - n_test_samples
+            native = Native.get_native_singleton()
+
             # Adapt test size if too small relative to number of classes
             if is_classification:
                 y_uniq = len(set(y))
-                n_test_samples = test_size if test_size >= 1 else len(y) * test_size
                 if n_test_samples < y_uniq:  # pragma: no cover
                     warnings.warn(
                         "Too few samples per class, adapting test size to guarantee 1 sample per class."
                     )
-                    test_size = y_uniq
+                    n_test_samples = y_uniq
+                    n_train_samples = n_samples - n_test_samples
+
+                sampling_result = native.stratified_sampling_without_replacement(
+                    random_state,
+                    y_uniq,
+                    n_train_samples,
+                    n_test_samples,
+                    y
+                )
+            else:
+                sampling_result = native.sample_without_replacement(
+                    random_state,
+                    n_train_samples,
+                    n_test_samples
+                )
 
-            # PaulK NOTE: sklearn train_test_split doesn't accept negative random_states
-            # we can remove the conversion to just positive values when we transition to C++
-            X_train, X_val, y_train, y_val, w_train, w_val = train_test_split(
-                X,
-                y,
-                w,
-                test_size=test_size,
-                random_state=(random_state - (-2147483648)) if random_state < 0 else random_state,
-                stratify=y if is_classification else None,
-            )
         else:  # pragma: no cover
-            raise Exception("test_size must be between 0 and 1.")
+            raise Exception("test_size must be a positive numeric value.")
+
+        if sampling_result is not None:
+            train_indices = np.where(sampling_result == 1)
+            test_indices = np.where(sampling_result == -1)
+            X_train = X[train_indices]
+            X_val = X[test_indices]
+            y_train = y[train_indices]
+            y_val = y[test_indices]
+            w_train = w[train_indices]
+            w_val = w[test_indices]
 
         if not is_train:
             X_train, y_train = None, None
@@ -469,3 +508,301 @@ class EBMUtils:
             # TODO PK we should consider changing the feature type to the same " x " separator
             # style as gen_feature_name, for human understanability
             return "interaction"
+
+    @staticmethod
+    def cyclic_gradient_boost(
+        model_type,
+        n_classes,
+        features_categorical, 
+        features_bin_count,
+        feature_groups,
+        X_train,
+        y_train,
+        w_train,
+        scores_train,
+        X_val,
+        y_val,
+        w_val,
+        scores_val,
+        n_inner_bags,
+        generate_update_options,
+        learning_rate,
+        min_samples_leaf,
+        max_leaves,
+        early_stopping_rounds,
+        early_stopping_tolerance,
+        max_rounds,
+        random_state,
+        name,
+        noise_scale,
+        bin_counts,
+        optional_temp_params=None,
+    ):
+        min_metric = np.inf
+        episode_index = 0
+        with Booster(
+            model_type,
+            n_classes,
+            features_categorical, 
+            features_bin_count,
+            feature_groups,
+            X_train,
+            y_train,
+            w_train,
+            scores_train,
+            X_val,
+            y_val,
+            w_val,
+            scores_val,
+            n_inner_bags,
+            random_state,
+            optional_temp_params,
+        ) as booster:
+            no_change_run_length = 0
+            bp_metric = np.inf
+            log.info("Start boosting {0}".format(name))
+            for episode_index in range(max_rounds):
+                if episode_index % 10 == 0:
+                    log.debug("Sweep Index for {0}: {1}".format(name, episode_index))
+                    log.debug("Metric: {0}".format(min_metric))
+
+                for feature_group_index in range(len(feature_groups)):
+                    gain = booster.generate_model_update(
+                        feature_group_index=feature_group_index,
+                        generate_update_options=generate_update_options,
+                        learning_rate=learning_rate,
+                        min_samples_leaf=min_samples_leaf,
+                        max_leaves=max_leaves,
+                    )
+
+                    if noise_scale: # Differentially private updates
+                        splits = booster.get_model_update_splits()[0]
+
+                        model_update_tensor = booster.get_model_update_expanded()
+                        noisy_update_tensor = model_update_tensor.copy()
+
+                        splits_iter = [0] + list(splits + 1) + [len(model_update_tensor)] # Make splits iteration friendly
+                        # Loop through all random splits and add noise before updating
+                        for f, s in zip(splits_iter[:-1], splits_iter[1:]):
+                            if s == 1: 
+                                continue # Skip cuts that fall on 0th (missing value) bin -- missing values not supported in DP
+
+                            noise = np.random.normal(0.0, noise_scale)
+                            noisy_update_tensor[f:s] = model_update_tensor[f:s] + noise
+
+                            # Native code will be returning sums of residuals in slices, not averages.
+                            # Compute noisy average by dividing noisy sum by noisy histogram counts
+                            instance_count = np.sum(bin_counts[feature_group_index][f:s])
+                            noisy_update_tensor[f:s] = noisy_update_tensor[f:s] / instance_count
+
+                        noisy_update_tensor = noisy_update_tensor * -1 # Invert gradients before updates
+                        booster.set_model_update_expanded(feature_group_index, noisy_update_tensor)
+
+
+                    curr_metric = booster.apply_model_update()
+
+                    min_metric = min(curr_metric, min_metric)
+
+                # TODO PK this early_stopping_tolerance is a little inconsistent
+                #      since it triggers intermittently and only re-triggers if the
+                #      threshold is re-passed, but not based on a smooth windowed set
+                #      of checks.  We can do better by keeping a list of the last
+                #      number of measurements to have a consistent window of values.
+                #      If we only cared about the metric at the start and end of the epoch
+                #      window a circular buffer would be best choice with O(1).
+                if no_change_run_length == 0:
+                    bp_metric = min_metric
+                if min_metric + early_stopping_tolerance < bp_metric:
+                    no_change_run_length = 0
+                else:
+                    no_change_run_length += 1
+
+                if (
+                    early_stopping_rounds >= 0
+                    and no_change_run_length >= early_stopping_rounds
+                ):
+                    break
+
+            log.info(
+                "End boosting {0}, Best Metric: {1}, Num Rounds: {2}".format(
+                    name, min_metric, episode_index
+                )
+            )
+
+            # TODO: Add more ways to call alternative get_current_model
+            # Use latest model if there are no instances in the (transposed) validation set 
+            # or if training with privacy
+            if X_val.shape[1] == 0 or noise_scale is not None:
+                model_update = booster.get_current_model()
+            else:
+                model_update = booster.get_best_model()
+
+        return model_update, min_metric, episode_index
+
+    @staticmethod
+    def get_interactions(
+        n_interactions,
+        iter_feature_groups,
+        model_type,
+        n_classes,
+        features_categorical, 
+        features_bin_count,
+        X,
+        y,
+        w,
+        scores,
+        min_samples_leaf,
+        optional_temp_params=None,
+    ):
+        interaction_scores = []
+        with InteractionDetector(
+            model_type, n_classes, features_categorical, features_bin_count, X, y, w, scores, optional_temp_params
+        ) as interaction_detector:
+            for feature_group in iter_feature_groups:
+                score = interaction_detector.get_interaction_score(
+                    feature_group, min_samples_leaf,
+                )
+                interaction_scores.append((feature_group, score))
+
+        ranked_scores = list(
+            sorted(interaction_scores, key=lambda x: x[1], reverse=True)
+        )
+        final_ranked_scores = ranked_scores
+
+        final_indices = [x[0] for x in final_ranked_scores]
+        final_scores = [x[1] for x in final_ranked_scores]
+
+        return final_indices, final_scores
+
+
+class DPUtils:
+
+    @staticmethod
+    def calc_classic_noise_multi(total_queries, target_epsilon, delta, sensitivity):
+        variance = (8*total_queries*sensitivity**2 * np.log(np.exp(1) + target_epsilon / delta)) / target_epsilon ** 2
+        return np.sqrt(variance)
+
+    @staticmethod
+    def calc_gdp_noise_multi(total_queries, target_epsilon, delta):
+        ''' GDP analysis following Algorithm 2 in: https://arxiv.org/abs/2106.09680. 
+        '''
+        def f(mu, eps, delta):
+            return DPUtils.delta_eps_mu(eps, mu) - delta
+
+        final_mu = brentq(lambda x: f(x, target_epsilon, delta), 1e-5, 1000)
+        sigma = np.sqrt(total_queries) / final_mu
+        return sigma
+
+    # General calculations, largely borrowed from tensorflow/privacy and presented in https://arxiv.org/abs/1911.11607
+    @staticmethod
+    def delta_eps_mu(eps, mu):
+        ''' Code adapted from: https://github.com/tensorflow/privacy/blob/master/tensorflow_privacy/privacy/analysis/gdp_accountant.py#L44
+        '''
+        return norm.cdf(-eps/mu + mu/2) - np.exp(eps) * norm.cdf(-eps/mu - mu/2)
+
+    @staticmethod
+    def eps_from_mu(mu, delta):
+        ''' Code adapted from: https://github.com/tensorflow/privacy/blob/master/tensorflow_privacy/privacy/analysis/gdp_accountant.py#L50
+        '''
+        def f(x):
+            return DPUtils.delta_eps_mu(x, mu)-delta    
+        return root_scalar(f, bracket=[0, 500], method='brentq').root
+
+    @staticmethod
+    def private_numeric_binning(col_data, noise_scale, max_bins, min_val, max_val):
+        uniform_counts, uniform_edges = np.histogram(col_data, bins=max_bins*2, range=(min_val, max_val))
+        noisy_counts = uniform_counts + np.random.normal(0, noise_scale, size=uniform_counts.shape[0])
+        
+        # Postprocess to ensure realistic bin values (integers, min=1)
+        noisy_counts = np.clip(np.round(noisy_counts), 1, None)
+
+        # Greedily collapse bins until they meet or exceed target_count threshold
+        target_count = col_data.shape[0] / max_bins
+        bin_counts, bin_cuts = [0], [uniform_edges[0]]
+        curr_count = 0
+        for index, right_edge in enumerate(uniform_edges[1:]):
+            curr_count += noisy_counts[index]
+            if curr_count >= target_count:
+                bin_cuts.append(right_edge)
+                bin_counts.append(curr_count)
+                curr_count = 0
+
+        # Ignore min/max value as part of cut definition
+        bin_cuts = np.array(bin_cuts)[1:-1] 
+
+        # All leftover datapoints get collapsed into final bin
+        bin_counts[-1] += curr_count
+
+        return bin_cuts, bin_counts
+
+    @staticmethod
+    def private_categorical_binning(col_data, noise_scale, max_bins):
+        # Initialize estimate
+        col_data = col_data.astype('U')
+        uniq_vals, counts = np.unique(col_data, return_counts=True)
+
+        counts = counts + np.random.normal(0, noise_scale, size=counts.shape[0])
+        counts = np.clip(np.round(counts), 1, None) # Postprocessing: Clip and round counts to be realistic (positive ints)
+
+        # Collapse bins until target_size is achieved.
+        target_count = col_data.shape[0] / max_bins
+        small_bins = np.where(counts < target_count)[0]
+        if len(small_bins) > 0:
+            other_count = np.sum(counts[small_bins])
+            mask = np.ones(counts.shape, dtype=bool)
+            mask[small_bins] = False
+
+            # Collapse all small bins into "DPOther"
+            uniq_vals = np.append(uniq_vals[mask], "DPOther")
+            counts = np.append(counts[mask], other_count)
+
+            # # If "DPOther" bin is too small, absorb 1 more bin (guaranteed above threshold)
+            if other_count < target_count:                               
+                collapse_bin = np.argmin(counts[:-1])
+                mask = np.ones(counts.shape, dtype=bool)
+                mask[collapse_bin] = False
+
+                # Pack data into the final "DPOther" bin
+                counts[-1] += counts[collapse_bin]
+
+                # Delete absorbed bin
+                uniq_vals = uniq_vals[mask]
+                counts = counts[mask]
+
+        return uniq_vals, counts
+
+    @staticmethod
+    def build_privacy_schema(X, y=None):
+        privacy_schema = {}
+        for index in range(X.shape[1]):
+            min_val, max_val = (np.min(X[:, index]), np.max(X[:, index]))
+            if isinstance(min_val, numbers.Number) and isinstance(max_val, numbers.Number):
+                privacy_schema[index] = (min_val, max_val)
+
+        if y is not None:
+            privacy_schema['target'] = (np.min(y), np.max(y))
+
+        return privacy_schema
+
+    @staticmethod
+    def validate_eps_delta(eps, delta):
+        if eps is None or eps <= 0 or delta is None or delta <= 0:
+            raise ValueError(f"Epsilon: '{eps}' and delta: '{delta}' must be set to positive numbers")
+
+    @staticmethod
+    def validate_DP_EBM(ebm):
+        fixed_params = dict(
+            max_interaction_bins=None,
+            interactions=0,
+            inner_bags=0,
+            early_stopping_rounds=-1,
+            early_stopping_tolerance=-1,
+        )
+
+        error_params = []
+        for param in fixed_params:
+            if getattr(ebm, param) != fixed_params[param]:
+                error_params.append(param)
+
+        if len(error_params) > 0:
+            raise ValueError(f"The following parameters: {error_params} cannot be changed when training with differential privacy.")
