@@ -19,7 +19,7 @@
 #include "compute_accessors.hpp"
 
 #include "RandomStream.hpp"
-#include "SegmentedTensor.hpp"
+#include "CompressibleTensor.hpp"
 #include "ebm_stats.hpp"
 // feature includes
 #include "Feature.hpp"
@@ -31,6 +31,7 @@
 #include "SamplingSet.hpp"
 #include "TreeSweep.hpp"
 
+#include "BoosterShell.hpp"
 #include "BoosterCore.hpp"
 
 namespace DEFINED_ZONE_NAME {
@@ -38,7 +39,7 @@ namespace DEFINED_ZONE_NAME {
 #error DEFINED_ZONE_NAME must be defined
 #endif // DEFINED_ZONE_NAME
 
-extern bool InitializeGradientsAndHessians(
+extern ErrorEbmType InitializeGradientsAndHessians(
    const ptrdiff_t runtimeLearningTypeOrCountTargetClasses,
    const size_t cSamples,
    const void * const aTargetData,
@@ -51,65 +52,68 @@ INLINE_ALWAYS static size_t GetCountItemsBitPacked(const size_t cBits) {
    return k_cBitsForStorageType / cBits;
 }
 
-void BoosterCore::DeleteSegmentedTensors(const size_t cFeatureGroups, SegmentedTensor ** const apSegmentedTensors) {
-   LOG_0(TraceLevelInfo, "Entered DeleteSegmentedTensors");
+void BoosterCore::DeleteCompressibleTensors(const size_t cFeatureGroups, CompressibleTensor ** const apCompressibleTensors) {
+   LOG_0(TraceLevelInfo, "Entered DeleteCompressibleTensors");
 
-   if(UNLIKELY(nullptr != apSegmentedTensors)) {
+   if(UNLIKELY(nullptr != apCompressibleTensors)) {
       EBM_ASSERT(0 < cFeatureGroups);
-      SegmentedTensor ** ppSegmentedTensors = apSegmentedTensors;
-      const SegmentedTensor * const * const ppSegmentedTensorsEnd = &apSegmentedTensors[cFeatureGroups];
+      CompressibleTensor ** ppCompressibleTensors = apCompressibleTensors;
+      const CompressibleTensor * const * const ppCompressibleTensorsEnd = &apCompressibleTensors[cFeatureGroups];
       do {
-         SegmentedTensor::Free(*ppSegmentedTensors);
-         ++ppSegmentedTensors;
-      } while(ppSegmentedTensorsEnd != ppSegmentedTensors);
-      free(apSegmentedTensors);
+         CompressibleTensor::Free(*ppCompressibleTensors);
+         ++ppCompressibleTensors;
+      } while(ppCompressibleTensorsEnd != ppCompressibleTensors);
+      free(apCompressibleTensors);
    }
-   LOG_0(TraceLevelInfo, "Exited DeleteSegmentedTensors");
+   LOG_0(TraceLevelInfo, "Exited DeleteCompressibleTensors");
 }
 
-SegmentedTensor ** BoosterCore::InitializeSegmentedTensors(
+ErrorEbmType BoosterCore::InitializeCompressibleTensors(
    const size_t cFeatureGroups, 
    const FeatureGroup * const * const apFeatureGroups, 
-   const size_t cVectorLength) 
+   const size_t cVectorLength,
+   CompressibleTensor *** papCompressibleTensorsOut)
 {
-   LOG_0(TraceLevelInfo, "Entered InitializeSegmentedTensors");
+   LOG_0(TraceLevelInfo, "Entered InitializeCompressibleTensors");
 
    EBM_ASSERT(0 < cFeatureGroups);
    EBM_ASSERT(nullptr != apFeatureGroups);
    EBM_ASSERT(1 <= cVectorLength);
+   EBM_ASSERT(nullptr != papCompressibleTensorsOut);
+   EBM_ASSERT(nullptr == *papCompressibleTensorsOut);
 
-   SegmentedTensor ** const apSegmentedTensors = EbmMalloc<SegmentedTensor *>(cFeatureGroups);
-   if(UNLIKELY(nullptr == apSegmentedTensors)) {
-      LOG_0(TraceLevelWarning, "WARNING InitializeSegmentedTensors nullptr == apSegmentedTensors");
-      return nullptr;
+   CompressibleTensor ** const apCompressibleTensors = EbmMalloc<CompressibleTensor *>(cFeatureGroups);
+   if(UNLIKELY(nullptr == apCompressibleTensors)) {
+      LOG_0(TraceLevelWarning, "WARNING InitializeCompressibleTensors nullptr == apCompressibleTensors");
+      return Error_OutOfMemory;
    }
    for(size_t i = 0; i < cFeatureGroups; ++i) {
-      apSegmentedTensors[i] = nullptr;
+      apCompressibleTensors[i] = nullptr;
    }
+   *papCompressibleTensorsOut = apCompressibleTensors; // transfer ownership for future deletion
 
-   SegmentedTensor ** ppSegmentedTensors = apSegmentedTensors;
+   CompressibleTensor ** ppCompressibleTensors = apCompressibleTensors;
    for(size_t iFeatureGroup = 0; iFeatureGroup < cFeatureGroups; ++iFeatureGroup) {
       const FeatureGroup * const pFeatureGroup = apFeatureGroups[iFeatureGroup];
-      SegmentedTensor * const pSegmentedTensors = 
-         SegmentedTensor::Allocate(pFeatureGroup->GetCountSignificantDimensions(), cVectorLength);
-      if(UNLIKELY(nullptr == pSegmentedTensors)) {
-         LOG_0(TraceLevelWarning, "WARNING InitializeSegmentedTensors nullptr == pSegmentedTensors");
-         DeleteSegmentedTensors(cFeatureGroups, apSegmentedTensors);
-         return nullptr;
+      CompressibleTensor * const pCompressibleTensors = 
+         CompressibleTensor::Allocate(pFeatureGroup->GetCountSignificantDimensions(), cVectorLength);
+      if(UNLIKELY(nullptr == pCompressibleTensors)) {
+         LOG_0(TraceLevelWarning, "WARNING InitializeCompressibleTensors nullptr == pCompressibleTensors");
+         return Error_OutOfMemory;
+      }
+      *ppCompressibleTensors = pCompressibleTensors; // transfer ownership for future deletion
+
+      const ErrorEbmType error = pCompressibleTensors->Expand(pFeatureGroup);
+      if(Error_None != error) {
+         // already logged
+         return error;
       }
 
-      if(pSegmentedTensors->Expand(pFeatureGroup)) {
-         LOG_0(TraceLevelWarning, "WARNING InitializeSegmentedTensors pSegmentedTensors->Expand(pFeatureGroup)");
-         DeleteSegmentedTensors(cFeatureGroups, apSegmentedTensors);
-         return nullptr;
-      }
-
-      *ppSegmentedTensors = pSegmentedTensors;
-      ++ppSegmentedTensors;
+      ++ppCompressibleTensors;
    }
 
-   LOG_0(TraceLevelInfo, "Exited InitializeSegmentedTensors");
-   return apSegmentedTensors;
+   LOG_0(TraceLevelInfo, "Exited InitializeCompressibleTensors");
+   return Error_None;
 }
 
 void BoosterCore::Free(BoosterCore * const pBoosterCore) {
@@ -141,7 +145,8 @@ void TODO_removeThisThreadTest() {
    g_TODO_removeThisThreadTest = 1;
 }
 
-BoosterCore * BoosterCore::Create(
+ErrorEbmType BoosterCore::Create(
+   BoosterShell * const pBoosterShell,
    const SeedEbmType randomSeed,
    const ptrdiff_t runtimeLearningTypeOrCountTargetClasses,
    const size_t cFeatures,
@@ -169,48 +174,53 @@ BoosterCore * BoosterCore::Create(
 
    LOG_0(TraceLevelInfo, "Entered BoosterCore::Create");
 
-   try {
-      // TODO: eliminate this code I added to test that threads are available on the majority of our systems
-      std::thread testThread(TODO_removeThisThreadTest);
-      testThread.join();
-      if(0 == g_TODO_removeThisThreadTest) {
-         LOG_0(TraceLevelWarning, "WARNING BoosterCore::Create thread not started");
-         return nullptr;
-      }
-   } catch(...) {
-      LOG_0(TraceLevelWarning, "WARNING BoosterCore::Create thread start failed");
-      return nullptr;
-   }
-
-   LOG_0(TraceLevelInfo, "Entered BoosterCore::Create thread started");
+   //try {
+   //   // TODO: eliminate this code I added to test that threads are available on the majority of our systems
+   //   std::thread testThread(TODO_removeThisThreadTest);
+   //   testThread.join();
+   //   if(0 == g_TODO_removeThisThreadTest) {
+   //      LOG_0(TraceLevelWarning, "WARNING BoosterCore::Create thread not started");
+   //      return Error_UnexpectedInternal;
+   //   }
+   //} catch(const std::bad_alloc &) {
+   //   LOG_0(TraceLevelWarning, "WARNING BoosterCore::Create thread start out of memory");
+   //   return Error_OutOfMemory;
+   //} catch(...) {
+   //   // the C++ standard doesn't really seem to say what kind of exceptions we'd get for various errors, so
+   //   // about the best we can do is catch(...) since the exact exceptions seem to be implementation specific
+   //   LOG_0(TraceLevelWarning, "WARNING BoosterCore::Create thread start failed");
+   //   return Error_ThreadStartFailed;
+   //}
+   //LOG_0(TraceLevelInfo, "INFO BoosterCore::Create thread started");
 
    BoosterCore * pBoosterCore;
    try {
       pBoosterCore = new BoosterCore();
    } catch(const std::bad_alloc &) {
       LOG_0(TraceLevelWarning, "WARNING BoosterCore::Create Out of memory allocating BoosterCore");
-      return nullptr;
+      return Error_OutOfMemory;
    } catch(...) {
       LOG_0(TraceLevelWarning, "WARNING BoosterCore::Create Unknown error");
-      return nullptr;
+      return Error_UnexpectedInternal;
    }
    if(nullptr == pBoosterCore) {
       // this should be impossible since bad_alloc should have been thrown, but let's be untrusting
       LOG_0(TraceLevelWarning, "WARNING BoosterCore::Create nullptr == pBoosterCore");
-      return nullptr;
+      return Error_OutOfMemory;
    }
+   // give ownership of our object to pBoosterShell
+   pBoosterShell->SetBoosterCore(pBoosterCore);
 
    const size_t cVectorLength = GetVectorLength(runtimeLearningTypeOrCountTargetClasses);
 
    LOG_0(TraceLevelInfo, "BoosterCore::Create starting feature processing");
    if(0 != cFeatures) {
+      pBoosterCore->m_cFeatures = cFeatures;
       pBoosterCore->m_aFeatures = EbmMalloc<Feature>(cFeatures);
       if(nullptr == pBoosterCore->m_aFeatures) {
          LOG_0(TraceLevelWarning, "WARNING BoosterCore::Create nullptr == pBoosterCore->m_aFeatures");
-         BoosterCore::Free(pBoosterCore);
-         return nullptr;
+         return Error_OutOfMemory;
       }
-      pBoosterCore->m_cFeatures = cFeatures;
 
       const BoolEbmType * pFeatureCategorical = aFeaturesCategorical;
       const IntEbmType * pFeatureBinCount = aFeaturesBinCount;
@@ -219,18 +229,15 @@ BoosterCore * BoosterCore::Create(
          const IntEbmType countBins = *pFeatureBinCount;
          if(countBins < 0) {
             LOG_0(TraceLevelError, "ERROR BoosterCore::Create countBins cannot be negative");
-            BoosterCore::Free(pBoosterCore);
-            return nullptr;
+            return Error_IllegalParamValue;
          }
          if(0 == countBins && (0 != cTrainingSamples || 0 != cValidationSamples)) {
             LOG_0(TraceLevelError, "ERROR BoosterCore::Create countBins cannot be zero if either 0 < cTrainingSamples OR 0 < cValidationSamples");
-            BoosterCore::Free(pBoosterCore);
-            return nullptr;
+            return Error_IllegalParamValue;
          }
-         if(!IsNumberConvertable<size_t>(countBins)) {
+         if(IsConvertError<size_t>(countBins)) {
             LOG_0(TraceLevelWarning, "WARNING BoosterCore::Create countBins is too high for us to allocate enough memory");
-            BoosterCore::Free(pBoosterCore);
-            return nullptr;
+            return Error_IllegalParamValue;
          }
          const size_t cBins = static_cast<size_t>(countBins);
          if(0 == cBins) {
@@ -271,14 +278,12 @@ BoosterCore * BoosterCore::Create(
       pBoosterCore->m_apFeatureGroups = FeatureGroup::AllocateFeatureGroups(cFeatureGroups);
       if(UNLIKELY(nullptr == pBoosterCore->m_apFeatureGroups)) {
          LOG_0(TraceLevelWarning, "WARNING BoosterCore::Create 0 != m_cFeatureGroups && nullptr == m_apFeatureGroups");
-         BoosterCore::Free(pBoosterCore);
-         return nullptr;
+         return Error_OutOfMemory;
       }
 
       if(GetTreeSweepSizeOverflow(bClassification, cVectorLength)) {
          LOG_0(TraceLevelWarning, "WARNING BoosterCore::Create GetTreeSweepSizeOverflow(bClassification, cVectorLength)");
-         BoosterCore::Free(pBoosterCore);
-         return nullptr;
+         return Error_OutOfMemory;
       }
       const size_t cBytesPerTreeSweep = GetTreeSweepSize(bClassification, cVectorLength);
 
@@ -288,22 +293,21 @@ BoosterCore * BoosterCore::Create(
          const IntEbmType countDimensions = aFeatureGroupsDimensionCount[iFeatureGroup];
          if(countDimensions < 0) {
             LOG_0(TraceLevelError, "ERROR BoosterCore::Create countDimensions cannot be negative");
-            BoosterCore::Free(pBoosterCore);
-            return nullptr;
+            return Error_IllegalParamValue;
          }
-         if(!IsNumberConvertable<size_t>(countDimensions)) {
+         if(IsConvertError<size_t>(countDimensions)) {
             // if countDimensions exceeds the size of size_t, then we wouldn't be able to find it
             // in the array passed to us
             LOG_0(TraceLevelError, "ERROR BoosterCore::Create countDimensions is too high to index");
-            BoosterCore::Free(pBoosterCore);
-            return nullptr;
+            // you can't really have more than size_t countDimensions since each dimension is a feature
+            // in a feature group, and our caller can't really have more than size_t of those
+            return Error_IllegalParamValue;
          }
          const size_t cDimensions = static_cast<size_t>(countDimensions);
          FeatureGroup * const pFeatureGroup = FeatureGroup::Allocate(cDimensions, iFeatureGroup);
          if(nullptr == pFeatureGroup) {
             LOG_0(TraceLevelWarning, "WARNING BoosterCore::Create nullptr == pFeatureGroup");
-            BoosterCore::Free(pBoosterCore);
-            return nullptr;
+            return Error_OutOfMemory;
          }
          // assign our pointer directly to our array right now so that we can't loose the memory if we decide to exit due to an error below
          pBoosterCore->m_apFeatureGroups[iFeatureGroup] = pFeatureGroup;
@@ -315,8 +319,7 @@ BoosterCore * BoosterCore::Create(
          } else {
             if(nullptr == pFeatureGroupFeatureIndexes) {
                LOG_0(TraceLevelError, "ERROR BoosterCore::Create aFeatureGroupsFeatureIndexes is null when there are FeatureGroups with non-zero numbers of features");
-               BoosterCore::Free(pBoosterCore);
-               return nullptr;
+               return Error_IllegalParamValue;
             }
             size_t cEquivalentSplits = 1;
             size_t cTensorBins = 1;
@@ -326,20 +329,17 @@ BoosterCore * BoosterCore::Create(
                const IntEbmType indexFeatureInterop = *pFeatureGroupFeatureIndexes;
                if(indexFeatureInterop < 0) {
                   LOG_0(TraceLevelError, "ERROR BoosterCore::Create aFeatureGroupsFeatureIndexes value cannot be negative");
-                  BoosterCore::Free(pBoosterCore);
-                  return nullptr;
+                  return Error_IllegalParamValue;
                }
-               if(!IsNumberConvertable<size_t>(indexFeatureInterop)) {
+               if(IsConvertError<size_t>(indexFeatureInterop)) {
                   LOG_0(TraceLevelError, "ERROR BoosterCore::Create aFeatureGroupsFeatureIndexes value too big to reference memory");
-                  BoosterCore::Free(pBoosterCore);
-                  return nullptr;
+                  return Error_IllegalParamValue;
                }
                const size_t iFeature = static_cast<size_t>(indexFeatureInterop);
 
                if(cFeatures <= iFeature) {
                   LOG_0(TraceLevelError, "ERROR BoosterCore::Create aFeatureGroupsFeatureIndexes value must be less than the number of features");
-                  BoosterCore::Free(pBoosterCore);
-                  return nullptr;
+                  return Error_IllegalParamValue;
                }
 
                EBM_ASSERT(1 <= cFeatures);
@@ -356,8 +356,7 @@ BoosterCore * BoosterCore::Create(
                   if(IsMultiplyError(cTensorBins, cBins)) {
                      // if this overflows, we definetly won't be able to allocate it
                      LOG_0(TraceLevelWarning, "WARNING BoosterCore::Create IsMultiplyError(cTensorStates, cBins)");
-                     BoosterCore::Free(pBoosterCore);
-                     return nullptr;
+                     return Error_OutOfMemory;
                   }
                   cTensorBins *= cBins;
                   cEquivalentSplits *= cBins - 1; // we can only split between the bins
@@ -375,18 +374,16 @@ BoosterCore * BoosterCore::Create(
                if(k_cDimensionsMax < cSignificantDimensions) {
                   // if we try to run with more than k_cDimensionsMax we'll exceed our memory capacity, so let's exit here instead
                   LOG_0(TraceLevelWarning, "WARNING BoosterCore::Create k_cDimensionsMax < cSignificantDimensions");
-                  BoosterCore::Free(pBoosterCore);
-                  return nullptr;
+                  return Error_OutOfMemory;
                }
 
                size_t cBytesArrayEquivalentSplit;
                if(1 == cSignificantDimensions) {
-                  if(IsMultiplyError(cEquivalentSplits, cBytesPerTreeSweep)) {
-                     LOG_0(TraceLevelWarning, "WARNING BoosterCore::Create IsMultiplyError(cEquivalentSplits, cBytesPerTreeSweep)");
-                     BoosterCore::Free(pBoosterCore);
-                     return nullptr;
+                  if(IsMultiplyError(cBytesPerTreeSweep, cEquivalentSplits)) {
+                     LOG_0(TraceLevelWarning, "WARNING BoosterCore::Create IsMultiplyError(cBytesPerTreeSweep, cEquivalentSplits)");
+                     return Error_OutOfMemory;
                   }
-                  cBytesArrayEquivalentSplit = cEquivalentSplits * cBytesPerTreeSweep;
+                  cBytesArrayEquivalentSplit = cBytesPerTreeSweep * cEquivalentSplits;
                } else {
                   // TODO : someday add equal gain multidimensional randomized picking.  It's rather hard though with the existing sweep functions for 
                   // multidimensional right now
@@ -408,17 +405,15 @@ BoosterCore * BoosterCore::Create(
       } while(iFeatureGroup < cFeatureGroups);
 
       if(!bClassification || ptrdiff_t { 2 } <= runtimeLearningTypeOrCountTargetClasses) {
-         pBoosterCore->m_apCurrentModel = InitializeSegmentedTensors(cFeatureGroups, pBoosterCore->m_apFeatureGroups, cVectorLength);
-         if(nullptr == pBoosterCore->m_apCurrentModel) {
+         ErrorEbmType error = InitializeCompressibleTensors(cFeatureGroups, pBoosterCore->m_apFeatureGroups, cVectorLength, &pBoosterCore->m_apCurrentModel);
+         if(Error_None != error) {
             LOG_0(TraceLevelWarning, "WARNING BoosterCore::Create nullptr == m_apCurrentModel");
-            BoosterCore::Free(pBoosterCore);
-            return nullptr;
+            return error;
          }
-         pBoosterCore->m_apBestModel = InitializeSegmentedTensors(cFeatureGroups, pBoosterCore->m_apFeatureGroups, cVectorLength);
-         if(nullptr == pBoosterCore->m_apBestModel) {
+         error = InitializeCompressibleTensors(cFeatureGroups, pBoosterCore->m_apFeatureGroups, cVectorLength, &pBoosterCore->m_apBestModel);
+         if(Error_None != error) {
             LOG_0(TraceLevelWarning, "WARNING BoosterCore::Create nullptr == m_apBestModel");
-            BoosterCore::Free(pBoosterCore);
-            return nullptr;
+            return error;
          }
       }
    }
@@ -426,40 +421,40 @@ BoosterCore * BoosterCore::Create(
 
    pBoosterCore->m_cBytesArrayEquivalentSplitMax = cBytesArrayEquivalentSplitMax;
 
-   if(pBoosterCore->m_trainingSet.Initialize(
-      true, 
+   const ErrorEbmType error1 = pBoosterCore->m_trainingSet.Initialize(
+      true,
       bClassification,
       bClassification,
-      bClassification, 
-      cFeatureGroups, 
+      bClassification,
+      cFeatureGroups,
       pBoosterCore->m_apFeatureGroups,
-      cTrainingSamples, 
-      aTrainingBinnedData, 
-      aTrainingTargets, 
-      aTrainingPredictorScores, 
+      cTrainingSamples,
+      aTrainingBinnedData,
+      aTrainingTargets,
+      aTrainingPredictorScores,
       runtimeLearningTypeOrCountTargetClasses
-   )) {
+   );
+   if(Error_None != error1) {
       LOG_0(TraceLevelWarning, "WARNING BoosterCore::Create m_trainingSet.Initialize");
-      BoosterCore::Free(pBoosterCore);
-      return nullptr;
+      return error1;
    }
 
-   if(pBoosterCore->m_validationSet.Initialize(
-      !bClassification, 
+   const ErrorEbmType error2 = pBoosterCore->m_validationSet.Initialize(
+      !bClassification,
       false,
-      bClassification, 
-      bClassification, 
-      cFeatureGroups, 
+      bClassification,
+      bClassification,
+      cFeatureGroups,
       pBoosterCore->m_apFeatureGroups,
-      cValidationSamples, 
-      aValidationBinnedData, 
-      aValidationTargets, 
-      aValidationPredictorScores, 
+      cValidationSamples,
+      aValidationBinnedData,
+      aValidationTargets,
+      aValidationPredictorScores,
       runtimeLearningTypeOrCountTargetClasses
-   )) {
+   );
+   if(Error_None != error2) {
       LOG_0(TraceLevelWarning, "WARNING BoosterCore::Create m_validationSet.Initialize");
-      BoosterCore::Free(pBoosterCore);
-      return nullptr;
+      return error2;
    }
 
    pBoosterCore->m_randomStream.InitializeUnsigned(randomSeed, k_boosterRandomizationMix);
@@ -470,8 +465,7 @@ BoosterCore * BoosterCore::Create(
       pBoosterCore->m_apSamplingSets = SamplingSet::GenerateSamplingSets(&pBoosterCore->m_randomStream, &pBoosterCore->m_trainingSet, aTrainingWeights, cSamplingSets);
       if(UNLIKELY(nullptr == pBoosterCore->m_apSamplingSets)) {
          LOG_0(TraceLevelWarning, "WARNING BoosterCore::Create nullptr == m_apSamplingSets");
-         BoosterCore::Free(pBoosterCore);
-         return nullptr;
+         return Error_OutOfMemory;
       }
    }
 
@@ -481,15 +475,13 @@ BoosterCore * BoosterCore::Create(
       if(IsMultiplyError(sizeof(*aValidationWeights), cValidationSamples)) {
          LOG_0(TraceLevelWarning,
             "WARNING BoosterCore::Create IsMultiplyError(sizeof(*aValidationWeights), cValidationSamples)");
-         BoosterCore::Free(pBoosterCore);
-         return nullptr;
+         return Error_IllegalParamValue;
       }
       if(!CheckAllWeightsEqual(cValidationSamples, aValidationWeights)) {
          const FloatEbmType total = AddPositiveFloatsSafe(cValidationSamples, aValidationWeights);
          if(std::isnan(total) || std::isinf(total) || total <= FloatEbmType { 0 }) {
             LOG_0(TraceLevelWarning, "WARNING BoosterCore::Create std::isnan(total) || std::isinf(total) || total <= FloatEbmType { 0 }");
-            BoosterCore::Free(pBoosterCore);
-            return nullptr;
+            return Error_UserParamValue;
          }
          // if they were all zero then we'd ignore the weights param.  If there are negative numbers it might add
          // to zero though so check it after checking for negative
@@ -500,8 +492,7 @@ BoosterCore * BoosterCore::Create(
          FloatEbmType * pValidationWeightInternal = static_cast<FloatEbmType *>(malloc(cBytes));
          if(UNLIKELY(nullptr == pValidationWeightInternal)) {
             LOG_0(TraceLevelWarning, "WARNING BoosterCore::Create nullptr == pValidationWeightInternal");
-            BoosterCore::Free(pBoosterCore);
-            return nullptr;
+            return Error_OutOfMemory;
          }
          pBoosterCore->m_aValidationWeights = pValidationWeightInternal;
          memcpy(pValidationWeightInternal, aValidationWeights, cBytes);
@@ -510,23 +501,23 @@ BoosterCore * BoosterCore::Create(
 
    if(bClassification) {
       if(0 != cTrainingSamples) {
-         if(InitializeGradientsAndHessians(
+         const ErrorEbmType error = InitializeGradientsAndHessians(
             runtimeLearningTypeOrCountTargetClasses,
             cTrainingSamples,
             aTrainingTargets,
             aTrainingPredictorScores,
             pBoosterCore->m_trainingSet.GetGradientsAndHessiansPointer()
-         )) {
+         );
+         if(Error_None != error) {
             // error already logged
-            BoosterCore::Free(pBoosterCore);
-            return nullptr;
+            return error;
          }
       }
    } else {
       EBM_ASSERT(IsRegression(runtimeLearningTypeOrCountTargetClasses));
       if(0 != cTrainingSamples) {
 #ifndef NDEBUG
-         const bool isFailed = 
+         const ErrorEbmType error =
 #endif // NDEBUG
          InitializeGradientsAndHessians(
             k_regression,
@@ -535,11 +526,11 @@ BoosterCore * BoosterCore::Create(
             aTrainingPredictorScores,
             pBoosterCore->m_trainingSet.GetGradientsAndHessiansPointer()
          );
-         EBM_ASSERT(!isFailed);
+         EBM_ASSERT(Error_None == error); // InitializeGradientsAndHessians doesn't allocate on regression
       }
       if(0 != cValidationSamples) {
 #ifndef NDEBUG
-         const bool isFailed =
+         const ErrorEbmType error =
 #endif // NDEBUG
          InitializeGradientsAndHessians(
             k_regression,
@@ -548,7 +539,7 @@ BoosterCore * BoosterCore::Create(
             aValidationPredictorScores,
             pBoosterCore->m_validationSet.GetGradientsAndHessiansPointer()
          );
-         EBM_ASSERT(!isFailed);
+         EBM_ASSERT(Error_None == error); // InitializeGradientsAndHessians doesn't allocate on regression
       }
    }
 
@@ -556,7 +547,7 @@ BoosterCore * BoosterCore::Create(
    pBoosterCore->m_bestModelMetric = FloatEbmType { std::numeric_limits<FloatEbmType>::max() };
 
    LOG_0(TraceLevelInfo, "Exited BoosterCore::Create");
-   return pBoosterCore;
+   return Error_None;
 }
 
 } // DEFINED_ZONE_NAME
