@@ -13,7 +13,7 @@
 
 #include "ebm_internal.hpp"
 
-#include "SegmentedTensor.hpp"
+#include "CompressibleTensor.hpp"
 
 #include "HistogramTargetEntry.hpp"
 
@@ -29,8 +29,8 @@ void BoosterShell::Free(BoosterShell * const pBoosterShell) {
    LOG_0(TraceLevelInfo, "Entered BoosterShell::Free");
 
    if(nullptr != pBoosterShell) {
-      SegmentedTensor::Free(pBoosterShell->m_pSmallChangeToModelAccumulatedFromSamplingSets);
-      SegmentedTensor::Free(pBoosterShell->m_pSmallChangeToModelOverwriteSingleSamplingSet);
+      CompressibleTensor::Free(pBoosterShell->m_pSmallChangeToModelAccumulatedFromSamplingSets);
+      CompressibleTensor::Free(pBoosterShell->m_pSmallChangeToModelOverwriteSingleSamplingSet);
       free(pBoosterShell->m_aThreadByteBuffer1);
       free(pBoosterShell->m_aThreadByteBuffer2);
       free(pBoosterShell->m_aSumHistogramTargetEntry);
@@ -72,12 +72,12 @@ ErrorEbmType BoosterShell::FillAllocations() {
    const size_t cBytesPerItem = IsClassification(runtimeLearningTypeOrCountTargetClasses) ?
       sizeof(HistogramTargetEntry<true>) : sizeof(HistogramTargetEntry<false>);
 
-   m_pSmallChangeToModelAccumulatedFromSamplingSets = SegmentedTensor::Allocate(k_cDimensionsMax, cVectorLength);
+   m_pSmallChangeToModelAccumulatedFromSamplingSets = CompressibleTensor::Allocate(k_cDimensionsMax, cVectorLength);
    if(nullptr == m_pSmallChangeToModelAccumulatedFromSamplingSets) {
       goto failed_allocation;
    }
 
-   m_pSmallChangeToModelOverwriteSingleSamplingSet = SegmentedTensor::Allocate(k_cDimensionsMax, cVectorLength);
+   m_pSmallChangeToModelOverwriteSingleSamplingSet = CompressibleTensor::Allocate(k_cDimensionsMax, cVectorLength);
    if(nullptr == m_pSmallChangeToModelOverwriteSingleSamplingSet) {
       goto failed_allocation;
    }
@@ -117,20 +117,24 @@ failed_allocation:;
    return Error_OutOfMemory;
 }
 
-HistogramBucketBase * BoosterShell::GetHistogramBucketBase(const size_t cBytesRequired) {
+HistogramBucketBase * BoosterShell::GetHistogramBucketBase(size_t cBytesRequired) {
    HistogramBucketBase * aBuffer = m_aThreadByteBuffer1;
    if(UNLIKELY(m_cThreadByteBufferCapacity1 < cBytesRequired)) {
-      m_cThreadByteBufferCapacity1 = cBytesRequired << 1;
-      LOG_N(TraceLevelInfo, "Growing BoosterShell::ThreadByteBuffer1 to %zu", m_cThreadByteBufferCapacity1);
+      cBytesRequired <<= 1;
+      m_cThreadByteBufferCapacity1 = cBytesRequired;
+      LOG_N(TraceLevelInfo, "Growing BoosterShell::ThreadByteBuffer1 to %zu", cBytesRequired);
 
       free(aBuffer);
-      aBuffer = static_cast<HistogramBucketBase *>(EbmMalloc<void>(m_cThreadByteBufferCapacity1));
-      m_aThreadByteBuffer1 = aBuffer;
+      aBuffer = static_cast<HistogramBucketBase *>(EbmMalloc<void>(cBytesRequired));
+      m_aThreadByteBuffer1 = aBuffer; // store it before checking it incase it's null so that we don't free old memory
+      if(nullptr == aBuffer) {
+         LOG_0(TraceLevelWarning, "WARNING BoosterShell::GetHistogramBucketBase OutOfMemory");
+      }
    }
    return aBuffer;
 }
 
-bool BoosterShell::GrowThreadByteBuffer2(const size_t cByteBoundaries) {
+ErrorEbmType BoosterShell::GrowThreadByteBuffer2(const size_t cByteBoundaries) {
    // by adding cByteBoundaries and shifting our existing size, we do 2 things:
    //   1) we ensure that if we have zero size, we'll get some size that we'll get a non-zero size after the shift
    //   2) we'll always get back an odd number of items, which is good because we always have an odd number of TreeNodeChilden
@@ -145,11 +149,12 @@ bool BoosterShell::GrowThreadByteBuffer2(const size_t cByteBoundaries) {
    void * aBuffer = m_aThreadByteBuffer2;
    free(aBuffer);
    aBuffer = EbmMalloc<void>(m_cThreadByteBufferCapacity2);
-   m_aThreadByteBuffer2 = aBuffer;
+   m_aThreadByteBuffer2 = aBuffer; // store it before checking it incase it's null so that we don't free old memory
    if(UNLIKELY(nullptr == aBuffer)) {
-      return true;
+      LOG_0(TraceLevelWarning, "WARNING GrowThreadByteBuffer2 OutOfMemory");
+      return Error_OutOfMemory;
    }
-   return false;
+   return Error_None;
 }
 
 
@@ -244,30 +249,30 @@ static ErrorEbmType CreateBooster(
       LOG_0(TraceLevelError, "ERROR CreateBooster countInnerBags must be positive");
       return Error_UserParamValue;
    }
-   if(!IsNumberConvertable<size_t>(countFeatures)) {
+   if(IsConvertError<size_t>(countFeatures)) {
       // the caller should not have been able to allocate enough memory in "features" if this didn't fit in memory
-      LOG_0(TraceLevelError, "ERROR CreateBooster !IsNumberConvertable<size_t>(countFeatures)");
+      LOG_0(TraceLevelError, "ERROR CreateBooster IsConvertError<size_t>(countFeatures)");
       return Error_IllegalParamValue;
    }
-   if(!IsNumberConvertable<size_t>(countFeatureGroups)) {
+   if(IsConvertError<size_t>(countFeatureGroups)) {
       // the caller should not have been able to allocate enough memory in "aFeatureGroupsDimensionCount" if this didn't fit in memory
-      LOG_0(TraceLevelError, "ERROR CreateBooster !IsNumberConvertable<size_t>(countFeatureGroups)");
+      LOG_0(TraceLevelError, "ERROR CreateBooster IsConvertError<size_t>(countFeatureGroups)");
       return Error_IllegalParamValue;
    }
-   if(!IsNumberConvertable<size_t>(countTrainingSamples)) {
+   if(IsConvertError<size_t>(countTrainingSamples)) {
       // the caller should not have been able to allocate enough memory in "trainingTargets" if this didn't fit in memory
-      LOG_0(TraceLevelError, "ERROR CreateBooster !IsNumberConvertable<size_t>(countTrainingSamples)");
+      LOG_0(TraceLevelError, "ERROR CreateBooster IsConvertError<size_t>(countTrainingSamples)");
       return Error_IllegalParamValue;
    }
-   if(!IsNumberConvertable<size_t>(countValidationSamples)) {
+   if(IsConvertError<size_t>(countValidationSamples)) {
       // the caller should not have been able to allocate enough memory in "validationTargets" if this didn't fit in memory
-      LOG_0(TraceLevelError, "ERROR CreateBooster !IsNumberConvertable<size_t>(countValidationSamples)");
+      LOG_0(TraceLevelError, "ERROR CreateBooster IsConvertError<size_t>(countValidationSamples)");
       return Error_IllegalParamValue;
    }
-   if(!IsNumberConvertable<size_t>(countInnerBags)) {
+   if(IsConvertError<size_t>(countInnerBags)) {
       // this is just a warning since the caller doesn't pass us anything material, but if it's this high
       // then our allocation would fail since it can't even in pricipal fit into memory
-      LOG_0(TraceLevelWarning, "WARNING CreateBooster !IsNumberConvertable<size_t>(countInnerBags)");
+      LOG_0(TraceLevelWarning, "WARNING CreateBooster IsConvertError<size_t>(countInnerBags)");
       return Error_OutOfMemory;
    }
 
@@ -296,10 +301,8 @@ static ErrorEbmType CreateBooster(
       return Error_OutOfMemory;
    }
 
-   // TODO: pass in the pBoosterShell so that BoosterCore can immediately attach itself to the pBoosterShell
-   //       this is important in R and other languages that might want to exit with longjump because we can attach
-   //       the pBoosterShell object to a managed destructor that'll clean up all our memory allocations
-   BoosterCore * const pBoosterCore = BoosterCore::Create(
+   const ErrorEbmType error1 = BoosterCore::Create(
+      pBoosterShell,
       randomSeed,
       runtimeLearningTypeOrCountTargetClasses,
       cFeatures,
@@ -321,19 +324,16 @@ static ErrorEbmType CreateBooster(
       aValidationWeights,
       validationPredictorScores
    );
-   if(UNLIKELY(nullptr == pBoosterCore)) {
+   if(UNLIKELY(Error_None != error1)) {
       BoosterShell::Free(pBoosterShell);
       LOG_0(TraceLevelWarning, "WARNING CreateBooster pBoosterCore->Initialize");
-      return Error_OutOfMemory;
+      return error1;
    }
 
-   pBoosterShell->SetBoosterCore(pBoosterCore); // assume ownership of pBoosterCore
-
-   const ErrorEbmType error = pBoosterShell->FillAllocations();
-   if(Error_None != error) {
-      // don't free the pBoosterCore since pBoosterShell now owns it
+   const ErrorEbmType error2 = pBoosterShell->FillAllocations();
+   if(Error_None != error2) {
       BoosterShell::Free(pBoosterShell);
-      return error;
+      return error2;
    }
 
    *boosterHandleOut = pBoosterShell->GetHandle();
@@ -425,8 +425,8 @@ EBM_NATIVE_IMPORT_EXPORT_BODY ErrorEbmType EBM_NATIVE_CALLING_CONVENTION CreateC
       LOG_0(TraceLevelError, "ERROR CreateClassificationBooster countTargetClasses can't be zero unless there are no training and no validation cases");
       return Error_IllegalParamValue;
    }
-   if(!IsNumberConvertable<ptrdiff_t>(countTargetClasses)) {
-      LOG_0(TraceLevelWarning, "WARNING CreateClassificationBooster !IsNumberConvertable<ptrdiff_t>(countTargetClasses)");
+   if(IsConvertError<ptrdiff_t>(countTargetClasses)) {
+      LOG_0(TraceLevelWarning, "WARNING CreateClassificationBooster IsConvertError<ptrdiff_t>(countTargetClasses)");
       // we didn't run out of memory, but we will if we accept this and it's not worth making a new error code
       return Error_OutOfMemory;
    }
@@ -638,7 +638,7 @@ EBM_NATIVE_IMPORT_EXPORT_BODY ErrorEbmType EBM_NATIVE_CALLING_CONVENTION GetBest
       ,
       static_cast<void *>(boosterHandle),
       indexFeatureGroup,
-      modelFeatureGroupTensorOut
+      static_cast<void *>(modelFeatureGroupTensorOut)
    );
 
    BoosterShell * const pBoosterShell = BoosterShell::GetBoosterShellFromBoosterHandle(boosterHandle);
@@ -651,7 +651,7 @@ EBM_NATIVE_IMPORT_EXPORT_BODY ErrorEbmType EBM_NATIVE_CALLING_CONVENTION GetBest
       LOG_0(TraceLevelError, "ERROR GetBestModelFeatureGroup indexFeatureGroup must be positive");
       return Error_IllegalParamValue;
    }
-   if(!IsNumberConvertable<size_t>(indexFeatureGroup)) {
+   if(IsConvertError<size_t>(indexFeatureGroup)) {
       // we wouldn't have allowed the creation of an feature set larger than size_t
       LOG_0(TraceLevelError, "ERROR GetBestModelFeatureGroup indexFeatureGroup is too high to index");
       return Error_IllegalParamValue;
@@ -691,7 +691,7 @@ EBM_NATIVE_IMPORT_EXPORT_BODY ErrorEbmType EBM_NATIVE_CALLING_CONVENTION GetBest
       do {
          const size_t cBins = pFeatureGroupEntry->m_pFeature->GetCountBins();
          // we've allocated this memory, so it should be reachable, so these numbers should multiply
-         EBM_ASSERT(!IsMultiplyError(cBins, cValues));
+         EBM_ASSERT(!IsMultiplyError(cValues, cBins));
          cValues *= cBins;
          ++pFeatureGroupEntry;
       } while(pFeatureGroupEntryEnd != pFeatureGroupEntry);
@@ -702,7 +702,7 @@ EBM_NATIVE_IMPORT_EXPORT_BODY ErrorEbmType EBM_NATIVE_CALLING_CONVENTION GetBest
    //    2) If m_runtimeLearningTypeOrCountTargetClasses was either 1 or 0, but we checked for this above too
    EBM_ASSERT(nullptr != pBoosterCore->GetBestModel());
 
-   SegmentedTensor * const pBestModel = pBoosterCore->GetBestModel()[iFeatureGroup];
+   CompressibleTensor * const pBestModel = pBoosterCore->GetBestModel()[iFeatureGroup];
    EBM_ASSERT(nullptr != pBestModel);
    EBM_ASSERT(pBestModel->GetExpanded()); // the model should have been expanded at startup
    FloatEbmType * const pValues = pBestModel->GetValuePointer();
@@ -729,7 +729,7 @@ EBM_NATIVE_IMPORT_EXPORT_BODY ErrorEbmType EBM_NATIVE_CALLING_CONVENTION GetCurr
       ,
       static_cast<void *>(boosterHandle),
       indexFeatureGroup,
-      modelFeatureGroupTensorOut
+      static_cast<void *>(modelFeatureGroupTensorOut)
    );
 
    BoosterShell * const pBoosterShell = BoosterShell::GetBoosterShellFromBoosterHandle(boosterHandle);
@@ -742,7 +742,7 @@ EBM_NATIVE_IMPORT_EXPORT_BODY ErrorEbmType EBM_NATIVE_CALLING_CONVENTION GetCurr
       LOG_0(TraceLevelError, "ERROR GetCurrentModelFeatureGroup indexFeatureGroup must be positive");
       return Error_IllegalParamValue;
    }
-   if(!IsNumberConvertable<size_t>(indexFeatureGroup)) {
+   if(IsConvertError<size_t>(indexFeatureGroup)) {
       // we wouldn't have allowed the creation of an feature set larger than size_t
       LOG_0(TraceLevelError, "ERROR GetCurrentModelFeatureGroup indexFeatureGroup is too high to index");
       return Error_IllegalParamValue;
@@ -782,7 +782,7 @@ EBM_NATIVE_IMPORT_EXPORT_BODY ErrorEbmType EBM_NATIVE_CALLING_CONVENTION GetCurr
       do {
          const size_t cBins = pFeatureGroupEntry->m_pFeature->GetCountBins();
          // we've allocated this memory, so it should be reachable, so these numbers should multiply
-         EBM_ASSERT(!IsMultiplyError(cBins, cValues));
+         EBM_ASSERT(!IsMultiplyError(cValues, cBins));
          cValues *= cBins;
          ++pFeatureGroupEntry;
       } while(pFeatureGroupEntryEnd != pFeatureGroupEntry);
@@ -793,7 +793,7 @@ EBM_NATIVE_IMPORT_EXPORT_BODY ErrorEbmType EBM_NATIVE_CALLING_CONVENTION GetCurr
    //    2) If m_runtimeLearningTypeOrCountTargetClasses was either 1 or 0, but we checked for this above too
    EBM_ASSERT(nullptr != pBoosterCore->GetCurrentModel());
 
-   SegmentedTensor * const pCurrentModel = pBoosterCore->GetCurrentModel()[iFeatureGroup];
+   CompressibleTensor * const pCurrentModel = pBoosterCore->GetCurrentModel()[iFeatureGroup];
    EBM_ASSERT(nullptr != pCurrentModel);
    EBM_ASSERT(pCurrentModel->GetExpanded()); // the model should have been expanded at startup
    FloatEbmType * const pValues = pCurrentModel->GetValuePointer();
