@@ -172,7 +172,7 @@ EBM_NATIVE_IMPORT_EXPORT_BODY void EBM_NATIVE_CALLING_CONVENTION CutUniform(
       countPositiveInfinityRet = IntEbmType { 0 };
    } else {
       if(UNLIKELY(countSamples <= IntEbmType { 0 })) {
-         // if there's 1 sample, then we can't split it, but we'd still want to determine the min, max, etc
+         // if there's 1 sample, then we can't cut it, but we'd still want to determine the min, max, etc
          // so continue processing
 
          countMissingValuesRet = IntEbmType { 0 };
@@ -184,8 +184,8 @@ EBM_NATIVE_IMPORT_EXPORT_BODY void EBM_NATIVE_CALLING_CONVENTION CutUniform(
             LOG_0(TraceLevelError, "ERROR CutUniform countSamples < IntEbmType { 0 }");
          }
       } else {
-         if(UNLIKELY(!IsNumberConvertable<size_t>(countSamples))) {
-            LOG_0(TraceLevelWarning, "WARNING CutUniform !IsNumberConvertable<size_t>(countSamples)");
+         if(UNLIKELY(IsConvertError<size_t>(countSamples))) {
+            LOG_0(TraceLevelWarning, "WARNING CutUniform IsConvertError<size_t>(countSamples)");
 
             countMissingValuesRet = IntEbmType { 0 };
             minNonInfinityValueRet = FloatEbmType { 0 };
@@ -236,7 +236,7 @@ EBM_NATIVE_IMPORT_EXPORT_BODY void EBM_NATIVE_CALLING_CONVENTION CutUniform(
          const size_t cMissingValues = cSamplesIncludingMissingValues - cSamples;
          // this is guaranteed to work since the number of missing values can't exceed the number of original
          // samples, and samples came to us as an IntEbmType
-         EBM_ASSERT(IsNumberConvertable<IntEbmType>(cMissingValues));
+         EBM_ASSERT(!IsConvertError<IntEbmType>(cMissingValues));
          countMissingValuesRet = static_cast<IntEbmType>(cMissingValues);
 
          // our minValue and maxValue calculations below depend on there being at least 1 sample
@@ -263,8 +263,8 @@ EBM_NATIVE_IMPORT_EXPORT_BODY void EBM_NATIVE_CALLING_CONVENTION CutUniform(
                   goto exit_with_log;
                }
 
-               if(UNLIKELY(!IsNumberConvertable<size_t>(countCuts))) {
-                  LOG_0(TraceLevelWarning, "WARNING CutUniform !IsNumberConvertable<size_t>(countCuts)");
+               if(UNLIKELY(IsConvertError<size_t>(countCuts))) {
+                  LOG_0(TraceLevelWarning, "WARNING CutUniform IsConvertError<size_t>(countCuts)");
                   goto exit_with_log;
                }
                const size_t cCuts = static_cast<size_t>(countCuts);
@@ -283,7 +283,7 @@ EBM_NATIVE_IMPORT_EXPORT_BODY void EBM_NATIVE_CALLING_CONVENTION CutUniform(
                // we check that cCuts can be multiplied with sizeof(*cutsLowerBoundInclusiveOut), and since
                // there is no way an element of cutsLowerBoundInclusiveOut is as small as 1 byte, we should
                // be able to add one to cCuts
-               EBM_ASSERT(!IsAddError(cCuts, size_t { 1 }));
+               EBM_ASSERT(!IsAddError(size_t { 1 }, cCuts));
                const size_t cBins = cCuts + size_t { 1 };
 
                const FloatEbmType cBinsFloat = static_cast<FloatEbmType>(cBins);
@@ -295,7 +295,7 @@ EBM_NATIVE_IMPORT_EXPORT_BODY void EBM_NATIVE_CALLING_CONVENTION CutUniform(
 
                      // if you have 2 internal bins it would be close to an overflow on the subtraction 
                      // of the divided values.  With 3 bins it isn't obvious to me how you'd get an
-                     // overflow after dividing it up into separate divisions.  So, let's assume
+                     // overflow after dividing it up into separate segments.  So, let's assume
                      // that 2 == cBins, so we can just take the average and report one cut
                      const FloatEbmType avg = ArithmeticMean(minValue, maxValue);
                      *cutsLowerBoundInclusiveOut = avg;
