@@ -14,7 +14,7 @@
 
 #include "ebm_internal.hpp"
 
-#include "SegmentedTensor.hpp"
+#include "CompressibleTensor.hpp"
 #include "ebm_stats.hpp"
 // feature includes
 #include "Feature.hpp"
@@ -101,11 +101,9 @@ static ErrorEbmType BoostZeroDimensional(
    }
    const size_t cBytesPerHistogramBucket = GetHistogramBucketSize(bClassification, cVectorLength);
 
-   HistogramBucketBase * const pHistogramBucket =
-      pBoosterShell->GetHistogramBucketBase(cBytesPerHistogramBucket);
-
+   HistogramBucketBase * const pHistogramBucket = pBoosterShell->GetHistogramBucketBase(cBytesPerHistogramBucket);
    if(UNLIKELY(nullptr == pHistogramBucket)) {
-      LOG_0(TraceLevelWarning, "WARNING nullptr == pHistogramBucket");
+      // already logged
       return Error_OutOfMemory;
    }
 
@@ -125,7 +123,7 @@ static ErrorEbmType BoostZeroDimensional(
       pTrainingSet
    );
 
-   SegmentedTensor * const pSmallChangeToModelOverwriteSingleSamplingSet = 
+   CompressibleTensor * const pSmallChangeToModelOverwriteSingleSamplingSet = 
       pBoosterShell->GetOverwritableModelUpdate();
    FloatEbmType * aValues = pSmallChangeToModelOverwriteSingleSamplingSet->GetValuePointer();
    if(bClassification) {
@@ -203,7 +201,7 @@ static ErrorEbmType BoostSingleDimensional(
 
    EBM_ASSERT(IntEbmType { 2 } <= countLeavesMax); // otherwise we would have called BoostZeroDimensional
    size_t cLeavesMax = static_cast<size_t>(countLeavesMax);
-   if(!IsNumberConvertable<size_t>(countLeavesMax)) {
+   if(IsConvertError<size_t>(countLeavesMax)) {
       // we can never exceed a size_t number of leaves, so let's just set it to the maximum if we were going to overflow because it will generate 
       // the same results as if we used the true number
       cLeavesMax = std::numeric_limits<size_t>::max();
@@ -219,16 +217,16 @@ static ErrorEbmType BoostSingleDimensional(
       return Error_OutOfMemory;
    }
    const size_t cBytesPerHistogramBucket = GetHistogramBucketSize(bClassification, cVectorLength);
-   if(IsMultiplyError(cHistogramBuckets, cBytesPerHistogramBucket)) {
+   if(IsMultiplyError(cBytesPerHistogramBucket, cHistogramBuckets)) {
       // TODO : move this to initialization where we execute it only once
-      LOG_0(TraceLevelWarning, "WARNING IsMultiplyError(cHistogramBuckets, cBytesPerHistogramBucket)");
+      LOG_0(TraceLevelWarning, "WARNING IsMultiplyError(cBytesPerHistogramBucket, cHistogramBuckets)");
       return Error_OutOfMemory;
    }
-   const size_t cBytesBuffer = cHistogramBuckets * cBytesPerHistogramBucket;
+   const size_t cBytesBuffer = cBytesPerHistogramBucket * cHistogramBuckets;
 
    HistogramBucketBase * const aHistogramBuckets = pBoosterShell->GetHistogramBucketBase(cBytesBuffer);
    if(UNLIKELY(nullptr == aHistogramBuckets)) {
-      LOG_0(TraceLevelWarning, "WARNING BoostSingleDimensional nullptr == aHistogramBuckets");
+      // already logged
       return Error_OutOfMemory;
    }
 
@@ -299,7 +297,7 @@ static ErrorEbmType BoostSingleDimensional(
 }
 
 // TODO: for higher dimensional spaces, we need to add/subtract individual cells alot and the hessian isn't required (yet) in order to make decisions about
-//   where to cut.  For dimensions higher than 2, we might want to copy the tensor to a new tensor AFTER binning that keeps only the gradients and then 
+//   where to split.  For dimensions higher than 2, we might want to copy the tensor to a new tensor AFTER binning that keeps only the gradients and then 
 //    go back to our original tensor after splits to determine the hessian
 static ErrorEbmType BoostMultiDimensional(
    BoosterShell * const pBoosterShell,
@@ -358,16 +356,16 @@ static ErrorEbmType BoostMultiDimensional(
       return Error_OutOfMemory;
    }
    const size_t cBytesPerHistogramBucket = GetHistogramBucketSize(bClassification, cVectorLength);
-   if(IsMultiplyError(cTotalBuckets, cBytesPerHistogramBucket)) {
-      LOG_0(TraceLevelWarning, "WARNING BoostMultiDimensional IsMultiplyError(cTotalBuckets, cBytesPerHistogramBucket)");
+   if(IsMultiplyError(cBytesPerHistogramBucket, cTotalBuckets)) {
+      LOG_0(TraceLevelWarning, "WARNING BoostMultiDimensional IsMultiplyError(cBytesPerHistogramBucket, cTotalBuckets)");
       return Error_OutOfMemory;
    }
-   const size_t cBytesBuffer = cTotalBuckets * cBytesPerHistogramBucket;
+   const size_t cBytesBuffer = cBytesPerHistogramBucket * cTotalBuckets;
 
    // we don't need to free this!  It's tracked and reused by pBoosterShell
    HistogramBucketBase * const aHistogramBuckets = pBoosterShell->GetHistogramBucketBase(cBytesBuffer);
    if(UNLIKELY(nullptr == aHistogramBuckets)) {
-      LOG_0(TraceLevelWarning, "WARNING BoostMultiDimensional nullptr == aHistogramBuckets");
+      // already logged
       return Error_OutOfMemory;
    }
 
@@ -585,7 +583,7 @@ static ErrorEbmType BoostRandom(
    const IntEbmType * const aLeavesMax,
    FloatEbmType * const pTotalGain
 ) {
-   // THIS RANDOM CUT FUNCTION IS PRIMARILY USED FOR DIFFERENTIAL PRIVACY EBMs
+   // THIS RANDOM SPLIT FUNCTION IS PRIMARILY USED FOR DIFFERENTIAL PRIVACY EBMs
 
    LOG_0(TraceLevelVerbose, "Entered BoostRandom");
 
@@ -613,16 +611,16 @@ static ErrorEbmType BoostRandom(
       return Error_OutOfMemory;
    }
    const size_t cBytesPerHistogramBucket = GetHistogramBucketSize(bClassification, cVectorLength);
-   if(IsMultiplyError(cTotalBuckets, cBytesPerHistogramBucket)) {
-      LOG_0(TraceLevelWarning, "WARNING BoostRandom IsMultiplyError(cTotalBuckets, cBytesPerHistogramBucket)");
+   if(IsMultiplyError(cBytesPerHistogramBucket, cTotalBuckets)) {
+      LOG_0(TraceLevelWarning, "WARNING BoostRandom IsMultiplyError(cBytesPerHistogramBucket, cTotalBuckets)");
       return Error_OutOfMemory;
    }
-   const size_t cBytesBuffer = cTotalBuckets * cBytesPerHistogramBucket;
+   const size_t cBytesBuffer = cBytesPerHistogramBucket * cTotalBuckets;
 
    // we don't need to free this!  It's tracked and reused by pBoosterShell
    HistogramBucketBase * const aHistogramBuckets = pBoosterShell->GetHistogramBucketBase(cBytesBuffer);
    if(UNLIKELY(nullptr == aHistogramBuckets)) {
-      LOG_0(TraceLevelWarning, "WARNING BoostRandom nullptr == aHistogramBuckets");
+      // already logged
       return Error_OutOfMemory;
    }
 
@@ -762,7 +760,7 @@ static ErrorEbmType GenerateModelUpdateInternal(
                   "WARNING GenerateModelUpdateInternal cSamplesRequiredForChildSplitMin is ignored when doing random splitting"
                );
             }
-            // THIS RANDOM CUT OPTION IS PRIMARILY USED FOR DIFFERENTIAL PRIVACY EBMs
+            // THIS RANDOM SPLIT OPTION IS PRIMARILY USED FOR DIFFERENTIAL PRIVACY EBMs
 
             const ErrorEbmType error = BoostRandom(
                pBoosterShell,
@@ -910,17 +908,17 @@ static int g_cLogGenerateModelUpdateParametersMessages = 10;
 
 // TODO : change this so that our caller allocates the memory that contains the update, but this is complicated in various ways
 //        we don't want to just copy the internal tensor into the memory region that our caller provides, and we want to work with
-//        compressed representations of the SegmentedTensor object while we're building it, so we'll work within the memory the caller
+//        compressed representations of the CompressibleTensor object while we're building it, so we'll work within the memory the caller
 //        provides, but that means we'll potentially need more memory than the full tensor, and we'll need to put some header info
 //        at the start, so the caller can't treat this memory as a pure tensor.
 //        So:
 //          1) provide a function that returns the maximum memory needed.  A smart caller will call this once on each feature_group, 
 //             choose the max and allocate it once
-//          2) return a compressed complete SegmentedTensor to the caller inside an opaque memory region 
+//          2) return a compressed complete CompressibleTensor to the caller inside an opaque memory region 
 //             (return the exact size that we require to the caller for copying)
 //          3) if caller wants a simplified tensor, then they call a separate function that expands the tensor 
 //             and returns a pointer to the memory inside the opaque object
-//          4) ApplyModelUpdate will take an opaque SegmentedTensor, and expand it if needed
+//          4) ApplyModelUpdate will take an opaque CompressibleTensor, and expand it if needed
 //        The benefit of returning a compressed object is that we don't have to do the work of expanding it if the caller decides not to use it 
 //        (which might happen in greedy algorithms)
 //        The other benefit of returning a compressed object is that our caller can store/copy it faster
@@ -981,7 +979,7 @@ EBM_NATIVE_IMPORT_EXPORT_BODY ErrorEbmType EBM_NATIVE_CALLING_CONVENTION Generat
       LOG_0(TraceLevelError, "ERROR GenerateModelUpdate indexFeatureGroup must be positive");
       return Error_IllegalParamValue;
    }
-   if(!IsNumberConvertable<size_t>(indexFeatureGroup)) {
+   if(IsConvertError<size_t>(indexFeatureGroup)) {
       // we wouldn't have allowed the creation of an feature set larger than size_t
       if(LIKELY(nullptr != gainOut)) {
          *gainOut = FloatEbmType { 0 };
@@ -1022,7 +1020,7 @@ EBM_NATIVE_IMPORT_EXPORT_BODY ErrorEbmType EBM_NATIVE_CALLING_CONVENTION Generat
    size_t cSamplesRequiredForChildSplitMin = size_t { 1 }; // this is the min value
    if(IntEbmType { 1 } <= countSamplesRequiredForChildSplitMin) {
       cSamplesRequiredForChildSplitMin = static_cast<size_t>(countSamplesRequiredForChildSplitMin);
-      if(!IsNumberConvertable<size_t>(countSamplesRequiredForChildSplitMin)) {
+      if(IsConvertError<size_t>(countSamplesRequiredForChildSplitMin)) {
          // we can never exceed a size_t number of samples, so let's just set it to the maximum if we were going to overflow because it will generate 
          // the same results as if we used the true number
          cSamplesRequiredForChildSplitMin = std::numeric_limits<size_t>::max();
