@@ -16,7 +16,7 @@
 
 #include "ebm_internal.hpp"
 
-#include "SegmentedTensor.hpp"
+#include "CompressibleTensor.hpp"
 #include "ebm_stats.hpp"
 #include "BoosterShell.hpp"
 
@@ -39,7 +39,7 @@ namespace DEFINED_ZONE_NAME {
 template<bool bClassification>
 static void Flatten(
    const TreeNode<bClassification> * const pTreeNode,
-   ActiveDataType ** const ppDivisions, 
+   ActiveDataType ** const ppSplits, 
    FloatEbmType ** const ppValues, 
    const size_t cVectorLength
 ) {
@@ -49,12 +49,12 @@ static void Flatten(
       const size_t cBytesPerTreeNode = GetTreeNodeSize(bClassification, cVectorLength);
       const TreeNode<bClassification> * const pLeftChild = GetLeftTreeNodeChild<bClassification>(
          pTreeNode->AFTER_GetTreeNodeChildren(), cBytesPerTreeNode);
-      Flatten<bClassification>(pLeftChild, ppDivisions, ppValues, cVectorLength);
-      **ppDivisions = pTreeNode->AFTER_GetDivisionValue();
-      ++(*ppDivisions);
+      Flatten<bClassification>(pLeftChild, ppSplits, ppValues, cVectorLength);
+      **ppSplits = pTreeNode->AFTER_GetSplitValue();
+      ++(*ppSplits);
       const TreeNode<bClassification> * const pRightChild = GetRightTreeNodeChild<bClassification>(
          pTreeNode->AFTER_GetTreeNodeChildren(), cBytesPerTreeNode);
-      Flatten<bClassification>(pRightChild, ppDivisions, ppValues, cVectorLength);
+      Flatten<bClassification>(pRightChild, ppSplits, ppValues, cVectorLength);
    } else {
       FloatEbmType * pValuesCur = *ppValues;
       FloatEbmType * const pValuesNext = pValuesCur + cVectorLength;
@@ -94,9 +94,9 @@ static void Flatten(
    }
 }
 
-// TODO: it would be easy for us to implement a -1 lookback where we make the first cut, find the second cut, elimnate the first cut and try 
-//   again on that side, then re-examine the second cut again.  For mains this would be very quick we have found that 2-3 cuts are optimimum.  
-//   Probably 1 cut isn't very good since with 2 cuts we can localize a region of high gain in the center somewhere
+// TODO: it would be easy for us to implement a -1 lookback where we make the first split, find the second split, elimnate the first split and try 
+//   again on that side, then re-examine the second split again.  For mains this would be very quick we have found that 2-3 splits are optimimum.  
+//   Probably 1 split isn't very good since with 2 splits we can localize a region of high gain in the center somewhere
 
 template<ptrdiff_t compilerLearningTypeOrCountTargetClasses>
 static bool ExamineNodeForPossibleFutureSplittingAndDetermineBestSplitPoint(
@@ -418,13 +418,13 @@ static bool ExamineNodeForPossibleFutureSplittingAndDetermineBestSplitPoint(
 
    EBM_ASSERT(reinterpret_cast<const char *>(aHistogramBucket) <= reinterpret_cast<const char *>(BEST_pHistogramBucketEntry));
    EBM_ASSERT(0 == (reinterpret_cast<const char *>(BEST_pHistogramBucketEntry) - reinterpret_cast<const char *>(aHistogramBucket)) % cBytesPerHistogramBucket);
-   pTreeNode->AFTER_SetDivisionValue((reinterpret_cast<const char *>(BEST_pHistogramBucketEntry) - 
+   pTreeNode->AFTER_SetSplitValue((reinterpret_cast<const char *>(BEST_pHistogramBucketEntry) - 
       reinterpret_cast<const char *>(aHistogramBucket)) / cBytesPerHistogramBucket);
 
    LOG_N(
       TraceLevelVerbose,
-      "Exited ExamineNodeForPossibleFutureSplittingAndDetermineBestSplitPoint: divisionValue=%zu, nodeSplittingScore=%" FloatEbmTypePrintf,
-      static_cast<size_t>(pTreeNode->AFTER_GetDivisionValue()),
+      "Exited ExamineNodeForPossibleFutureSplittingAndDetermineBestSplitPoint: splitValue=%zu, nodeSplittingScore=%" FloatEbmTypePrintf,
+      static_cast<size_t>(pTreeNode->AFTER_GetSplitValue()),
       pTreeNode->AFTER_GetSplitGain()
    );
 
@@ -462,6 +462,8 @@ public:
    ) {
       constexpr bool bClassification = IsClassification(compilerLearningTypeOrCountTargetClasses);
 
+      ErrorEbmType error;
+
       HistogramBucketBase * const aHistogramBucketBase = pBoosterShell->GetHistogramBucketBase();
       const HistogramBucket<bClassification> * const aHistogramBucket =
          aHistogramBucketBase->GetHistogramBucket<bClassification>();
@@ -504,9 +506,10 @@ public:
          // TODO : we can eliminate this check as long as we ensure that the ThreadByteBuffer2 is always initialized to be equal to the size of three 
          // TreeNodes (left and right) == GET_SIZEOF_ONE_TREE_NODE_CHILDREN(cBytesPerTreeNode), or the number of bins (interactions multiply bins) on the 
          // highest bin count feature
-         if(pBoosterShell->GrowThreadByteBuffer2(cBytesInitialNeededAllocation)) {
-            LOG_0(TraceLevelWarning, "WARNING PartitionOneDimensionalBoosting pBoosterShell->GrowThreadByteBuffer2(cBytesInitialNeededAllocation)");
-            return Error_OutOfMemory;
+         error = pBoosterShell->GrowThreadByteBuffer2(cBytesInitialNeededAllocation);
+         if(Error_None != error) {
+            // already logged
+            return error;
          }
          cBytesBuffer2 = pBoosterShell->GetThreadByteBuffer2Size();
          EBM_ASSERT(cBytesInitialNeededAllocation <= cBytesBuffer2);
@@ -537,7 +540,7 @@ public:
          cVectorLength * sizeof(*aSumHistogramTargetEntry)
       );
 
-      SegmentedTensor * const pSmallChangeToModelOverwriteSingleSamplingSet =
+      CompressibleTensor * const pSmallChangeToModelOverwriteSingleSamplingSet =
          pBoosterShell->GetOverwritableModelUpdate();
 
       size_t cLeaves;
@@ -548,9 +551,10 @@ public:
          cSamplesRequiredForChildSplitMin
       )) {
          // there will be no splits at all
-         if(UNLIKELY(pSmallChangeToModelOverwriteSingleSamplingSet->SetCountDivisions(0, 0))) {
-            LOG_0(TraceLevelWarning, "WARNING PartitionOneDimensionalBoosting pSmallChangeToModelOverwriteSingleSamplingSet->SetCountDivisions(0, 0)");
-            return Error_OutOfMemory;
+         error = pSmallChangeToModelOverwriteSingleSamplingSet->SetCountSplits(0, 0);
+         if(UNLIKELY(Error_None != error)) {
+            // already logged
+            return error;
          }
 
          // we don't need to call EnsureValueCapacity because by default we start with a value capacity of 2 * cVectorLength
@@ -601,13 +605,14 @@ public:
                )->IsSplittable()
          );
 
-         if(UNLIKELY(pSmallChangeToModelOverwriteSingleSamplingSet->SetCountDivisions(0, 1))) {
-            LOG_0(TraceLevelWarning, "WARNING PartitionOneDimensionalBoosting pSmallChangeToModelOverwriteSingleSamplingSet->SetCountDivisions(0, 1)");
-            return Error_OutOfMemory;
+         error = pSmallChangeToModelOverwriteSingleSamplingSet->SetCountSplits(0, 1);
+         if(UNLIKELY(Error_None != error)) {
+            // already logged
+            return error;
          }
 
-         ActiveDataType * pDivisions = pSmallChangeToModelOverwriteSingleSamplingSet->GetDivisionPointer(0);
-         pDivisions[0] = pRootTreeNode->AFTER_GetDivisionValue();
+         ActiveDataType * pSplits = pSmallChangeToModelOverwriteSingleSamplingSet->GetSplitPointer(0);
+         pSplits[0] = pRootTreeNode->AFTER_GetSplitValue();
 
          // we don't need to call EnsureValueCapacity because by default we start with a value capacity of 2 * cVectorLength
 
@@ -718,9 +723,9 @@ public:
             // can have zero instnaces in bins, in which case it occurs, but those equivalent situations have been cleansed by
             // the time we reach this code, so the only realistic scenario where we might get equivalent gains is if we had an almost
             // symetric distribution samples bin distributions AND two tail ends that happen to have the same statistics AND
-            // either this is our first cut, or we've only made a single cut in the center in the case where there is symetry in the center
-            // Even if all of these things are true, after one non-symetric cut, we won't see that scenario anymore since the gradients won't be
-            // symetric anymore.  This is so rare, and limited to one cut, so we shouldn't bother to handle it since the complexity of doing so
+            // either this is our first split, or we've only made a single split in the center in the case where there is symetry in the center
+            // Even if all of these things are true, after one non-symetric split, we won't see that scenario anymore since the gradients won't be
+            // symetric anymore.  This is so rare, and limited to one split, so we shouldn't bother to handle it since the complexity of doing so
             // outweights the benefits.
             bestTreeNodeToSplit.pop();
 
@@ -744,9 +749,10 @@ public:
                   AddBytesTreeNode<bClassification>(pTreeNodeChildrenAvailableStorageSpaceCur, cBytesPerTreeNode << 1);
                if(cBytesBuffer2 <
                   static_cast<size_t>(reinterpret_cast<char *>(pTreeNodeChildrenAvailableStorageSpaceNext) - reinterpret_cast<char *>(pRootTreeNode))) {
-                  if(pBoosterShell->GrowThreadByteBuffer2(cBytesPerTreeNode)) {
-                     LOG_0(TraceLevelWarning, "WARNING PartitionOneDimensionalBoosting pBoosterShell->GrowThreadByteBuffer2(cBytesPerTreeNode)");
-                     return Error_OutOfMemory;
+                  error = pBoosterShell->GrowThreadByteBuffer2(cBytesPerTreeNode);
+                  if(Error_None != error) {
+                     // already logged
+                     return error;
                   }
                   goto retry_with_bigger_tree_node_children_array;
                }
@@ -787,9 +793,10 @@ public:
                   AddBytesTreeNode<bClassification>(pTreeNodeChildrenAvailableStorageSpaceCur, cBytesPerTreeNode << 1);
                if(cBytesBuffer2 <
                   static_cast<size_t>(reinterpret_cast<char *>(pTreeNodeChildrenAvailableStorageSpaceNext) - reinterpret_cast<char *>(pRootTreeNode))) {
-                  if(pBoosterShell->GrowThreadByteBuffer2(cBytesPerTreeNode)) {
-                     LOG_0(TraceLevelWarning, "WARNING PartitionOneDimensionalBoosting pBoosterShell->GrowThreadByteBuffer2(cBytesPerTreeNode)");
-                     return Error_OutOfMemory;
+                  error = pBoosterShell->GrowThreadByteBuffer2(cBytesPerTreeNode);
+                  if(Error_None != error) {
+                     // already logged
+                     return error;
                   }
                   goto retry_with_bigger_tree_node_children_array;
                }
@@ -845,27 +852,29 @@ public:
          return Error_UnexpectedInternal;
       }
 
-      if(UNLIKELY(pSmallChangeToModelOverwriteSingleSamplingSet->SetCountDivisions(0, cLeaves - size_t { 1 }))) {
-         LOG_0(TraceLevelWarning, "WARNING PartitionOneDimensionalBoosting pSmallChangeToModelOverwriteSingleSamplingSet->SetCountDivisions(0, cLeaves - size_t { 1 })");
-         return Error_OutOfMemory;
+      error = pSmallChangeToModelOverwriteSingleSamplingSet->SetCountSplits(0, cLeaves - size_t { 1 });
+      if(UNLIKELY(Error_None != error)) {
+         // already logged
+         return error;
       }
       if(IsMultiplyError(cVectorLength, cLeaves)) {
          LOG_0(TraceLevelWarning, "WARNING PartitionOneDimensionalBoosting IsMultiplyError(cVectorLength, cLeaves)");
          return Error_OutOfMemory;
       }
-      if(UNLIKELY(pSmallChangeToModelOverwriteSingleSamplingSet->EnsureValueCapacity(cVectorLength * cLeaves))) {
-         LOG_0(TraceLevelWarning, "WARNING PartitionOneDimensionalBoosting pSmallChangeToModelOverwriteSingleSamplingSet->EnsureValueCapacity(cVectorLength * cLeaves");
-         return Error_OutOfMemory;
+      error = pSmallChangeToModelOverwriteSingleSamplingSet->EnsureValueCapacity(cVectorLength * cLeaves);
+      if(UNLIKELY(Error_None != error)) {
+         // already logged
+         return error;
       }
-      ActiveDataType * pDivisions = pSmallChangeToModelOverwriteSingleSamplingSet->GetDivisionPointer(0);
+      ActiveDataType * pSplits = pSmallChangeToModelOverwriteSingleSamplingSet->GetSplitPointer(0);
       FloatEbmType * pValues = pSmallChangeToModelOverwriteSingleSamplingSet->GetValuePointer();
 
       LOG_0(TraceLevelVerbose, "Entered Flatten");
-      Flatten<bClassification>(pRootTreeNode, &pDivisions, &pValues, cVectorLength);
+      Flatten<bClassification>(pRootTreeNode, &pSplits, &pValues, cVectorLength);
       LOG_0(TraceLevelVerbose, "Exited Flatten");
 
-      EBM_ASSERT(pSmallChangeToModelOverwriteSingleSamplingSet->GetDivisionPointer(0) <= pDivisions);
-      EBM_ASSERT(static_cast<size_t>(pDivisions - pSmallChangeToModelOverwriteSingleSamplingSet->GetDivisionPointer(0)) == cLeaves - 1);
+      EBM_ASSERT(pSmallChangeToModelOverwriteSingleSamplingSet->GetSplitPointer(0) <= pSplits);
+      EBM_ASSERT(static_cast<size_t>(pSplits - pSmallChangeToModelOverwriteSingleSamplingSet->GetSplitPointer(0)) == cLeaves - 1);
       EBM_ASSERT(pSmallChangeToModelOverwriteSingleSamplingSet->GetValuePointer() < pValues);
       EBM_ASSERT(static_cast<size_t>(pValues - pSmallChangeToModelOverwriteSingleSamplingSet->GetValuePointer()) == cVectorLength * cLeaves);
 
